@@ -1,3 +1,16 @@
+## @file
+# Base file for Lemonldap::NG handlers
+
+## @class
+# Base class for Lemonldap::NG handlers.
+# All methods in handler are class methods: in ModPerl environment, handlers
+# are always launched without object created.
+#
+# The main method is run() who is called by Apache for each requests (using
+# handler() wrapper).
+#
+# The main initialization subroutine is init() who launch localInit() and
+# globalInit().
 package Lemonldap::NG::Handler::Simple;
 
 use strict;
@@ -5,42 +18,16 @@ use strict;
 use MIME::Base64;
 use Exporter 'import';
 use Safe;
+require Data::Dumper;
 require POSIX;
 
-our $VERSION = '0.84';
+our $VERSION = '0.9';
 
-our %EXPORT_TAGS = (
-    localStorage =>
-      [ qw( $localStorage $localStorageOptions $refLocalStorage ) ],
-    globalStorage => [ qw( $globalStorage $globalStorageOptions ) ],
-    locationRules => [
-        qw(
-          $locationCondition $defaultCondition $locationCount
-          $locationRegexp $apacheRequest $datas $safe $portal
-          $logout
-          )
-    ],
-    import  => [ qw( import @EXPORT_OK @EXPORT %EXPORT_TAGS ) ],
-    headers => [
-        qw(
-          $forgeHeaders lmHeaderIn lmSetHeaderIn lmHeaderOut
-          lmSetHeaderOut lmSetErrHeaderOut $cookieName $cookieSecured
-          $https $port
-          )
-    ],
-    traces => [ qw( $whatToTrace ) ],
-    apache =>
-      [ qw( MP OK REDIRECT FORBIDDEN DONE DECLINED SERVER_ERROR ) ],
-);
+our %EXPORT_TAGS;
 
-our @EXPORT_OK = ();
-push( @EXPORT_OK, @{ $EXPORT_TAGS{$_} } )
-  foreach (
-    qw( localStorage globalStorage locationRules import headers traces apache )
-  );
-$EXPORT_TAGS{all} = \@EXPORT_OK;
+our @EXPORT_OK;
 
-our @EXPORT = ();
+our @EXPORT;
 
 # Shared variables
 our (
@@ -50,7 +37,8 @@ our (
     $globalStorage,       $globalStorageOptions, $localStorage,
     $localStorageOptions, $whatToTrace,          $https,
     $refLocalStorage,     $safe,                 $cookieSecured,
-    $logout,              $port
+    $port,                $statusPipe,           $statusOut,
+    $customFunctions,
 );
 
 ##########################################
@@ -58,31 +46,66 @@ our (
 ##########################################
 
 BEGIN {
+    %EXPORT_TAGS = (
+        localStorage =>
+          [qw( $localStorage $localStorageOptions $refLocalStorage )],
+        globalStorage => [qw( $globalStorage $globalStorageOptions )],
+        locationRules => [
+            qw(
+              $locationCondition $defaultCondition $locationCount
+              $locationRegexp $apacheRequest $datas $safe $portal
+              safe $customFunctions
+              )
+        ],
+        import  => [qw( import @EXPORT_OK @EXPORT %EXPORT_TAGS )],
+        headers => [
+            qw(
+              $forgeHeaders lmHeaderIn lmSetHeaderIn lmHeaderOut
+              lmSetHeaderOut lmSetErrHeaderOut $cookieName $cookieSecured
+              $https $port
+              )
+        ],
+        log    => [qw(lmSetApacheUser)],
+        traces => [qw( $whatToTrace $statusPipe $statusOut )],
+        apache => [qw( MP OK REDIRECT FORBIDDEN DONE DECLINED SERVER_ERROR )],
+    );
+    push( @EXPORT_OK, @{ $EXPORT_TAGS{$_} } ) foreach ( keys %EXPORT_TAGS );
+    $EXPORT_TAGS{all} = \@EXPORT_OK;
     if ( exists $ENV{MOD_PERL} ) {
         if ( $ENV{MOD_PERL_API_VERSION} and $ENV{MOD_PERL_API_VERSION} >= 2 ) {
-            *MP = sub { 2 };
+            eval 'use constant MP => 2;';
         }
         else {
-            *MP = sub { 1 };
+            eval 'use constant MP => 1;';
         }
     }
     else {
-        *MP = sub { 0 };
+        eval 'use constant MP => 0;';
     }
     if ( MP() == 2 ) {
+        require Apache2::Log;
+        require Apache2::RequestUtil;
+        Apache2::RequestUtil->import();
         require Apache2::RequestRec;
         Apache2::RequestRec->import();
-        require Apache2::Log;
+        require Apache2::ServerUtil;
+        Apache2::ServerUtil->import();
+        require Apache2::Connection;
+        Apache2::Connection->import();
+        require Apache2::RequestIO;
+        Apache2::RequestIO->import();
+        require APR::Table;
+        APR::Table->import();
         require Apache2::Const;
         Apache2::Const->import( '-compile', qw(:common :log) );
-        *FORBIDDEN    = \&Apache2::Const::FORBIDDEN;
-        *REDIRECT     = \&Apache2::Const::REDIRECT;
-        *OK           = \&Apache2::Const::OK;
-        *DECLINED     = \&Apache2::Const::DECLINED;
-        *DONE         = \&Apache2::Const::DONE;
-        *SERVER_ERROR = \&Apache2::Const::SERVER_ERROR;
-        require Apache2::compat;
-        Apache2::compat->import();
+        eval '
+        use constant FORBIDDEN    => Apache2::Const::FORBIDDEN;
+        use constant REDIRECT     => Apache2::Const::REDIRECT;
+        use constant OK           => Apache2::Const::OK;
+        use constant DECLINED     => Apache2::Const::DECLINED;
+        use constant DONE         => Apache2::Const::DONE;
+        use constant SERVER_ERROR => Apache2::Const::SERVER_ERROR;
+        ';
         eval {
             require threads::shared;
             threads::shared::share($locationRegexp);
@@ -100,6 +123,8 @@ BEGIN {
             threads::shared::share($https);
             threads::shared::share($port);
             threads::shared::share($refLocalStorage);
+            threads::shared::share($statusPipe);
+            threads::shared::share($statusOut);
         };
     }
     elsif ( MP() == 1 ) {
@@ -111,24 +136,46 @@ BEGIN {
     }
     else {    # For Test or CGI
         eval '
-            sub OK {0}
-            sub FORBIDDEN {1}
-            sub REDIRECT {2}
-            sub DECLINED {1}
-            sub DONE {4}
-            sub SERVER_ERROR {5}
+        use constant FORBIDDEN    => 1;
+        use constant REDIRECT     => 1;
+        use constant OK           => 1;
+        use constant DECLINED     => 1;
+        use constant DONE         => 1;
+        use constant SERVER_ERROR => 1;
         ';
     }
     *handler = ( MP() == 2 ) ? \&handler_mp2 : \&handler_mp1;
-    *logout = ( MP() == 2 ) ? \&logout_mp2 : \&logout_mp1;
+    *logout  = ( MP() == 2 ) ? \&logout_mp2  : \&logout_mp1;
 }
 
-sub handler_mp1 ($$) { shift->run(@_) }
-sub handler_mp2 : method { shift->run(@_); }
+## @rmethod protected int handler_mp1()
+# Launch run() when used under mod_perl version 1
+# @return Apache constant
+sub handler_mp1 ($$) { shift->run(@_); }
 
-sub logout_mp1 ($$) { shift->unlog(@_) }
-sub logout_mp2 : method { shift->unlog(@_); }
+## @rmethod protected int handler_mp2()
+# Launch run() when used under mod_perl version 2
+# @return Apache constant
+sub handler_mp2 : method {
+    shift->run(@_);
+}
 
+## @rmethod protected int logout_mp1()
+# Launch unlog() when used under mod_perl version 1
+# @return Apache constant
+sub logout_mp1 ($$) { shift->unlog(@_); }
+
+## @rmethod protected int logout_mp2()
+# Launch unlog() when used under mod_perl version 2
+# @return Apache constant
+sub logout_mp2 : method {
+    shift->unlog(@_);
+}
+
+## @rmethod void lmLog(string mess, string level)
+# Wrapper for Apache log system
+# @param $mess message to log
+# @param $level string (debug, info, warning or error)
 sub lmLog {
     my ( $class, $mess, $level ) = @_;
     if ( MP() == 2 ) {
@@ -142,12 +189,41 @@ sub lmLog {
     }
 }
 
+## @rfn protected void lmSetApacheUser(Apache2::RequestRec r,string s)
+# Inform Apache for the data to use as user for logs
+# @param $r current request
+# @param $s string to use
+sub lmSetApacheUser {
+    my ( $r, $s ) = @_;
+    return unless ($s);
+    if ( MP() == 2 ) {
+        $r->user($s);
+    }
+    else {
+        $r->connection->user($s);
+    }
+}
+
+## @ifn protected string protected regRemoteIp(string str)
+# Replaces $ip by the client IP address in the string
+# @param $str string
+# @return string
 sub regRemoteIp {
     my ( $class, $str ) = @_;
-    $str =~ s/\$datas->\{ip\}/\$apacheRequest->connection->remote_ip/g;
+    if( MP() == 2 ) {
+        $str =~ s/\$datas->\{ip\}/\$apacheRequest->connection->remote_ip/g;
+    }
+    else {
+        $str =~ s/\$datas->\{ip\}/\$apacheRequest->remote_ip/g;
+    }
     return $str;
 }
 
+## @rfn void lmSetHeaderIn(Apache2::RequestRec r, string h, string v)
+# Set an HTTP header in the HTTP request.
+# @param $r Current request
+# @param $h Name of the header
+# @param $v Value of the header
 sub lmSetHeaderIn {
     my ( $r, $h, $v ) = @_;
     if ( MP() == 2 ) {
@@ -158,8 +234,13 @@ sub lmSetHeaderIn {
     }
 }
 
+## @rfn string lmtHeaderIn(Apache2::RequestRec r, string h)
+# Return an HTTP header value from the HTTP request.
+# @param $r Current request
+# @param $h Name of the header
+# @return Value of the header
 sub lmHeaderIn {
-    my ( $r, $h, $v ) = @_;
+    my ( $r, $h ) = @_;
     if ( MP() == 2 ) {
         return $r->headers_in->{$h};
     }
@@ -168,6 +249,11 @@ sub lmHeaderIn {
     }
 }
 
+## @rfn void lmSetErrHeaderOut(Apache2::RequestRec r, string h, string v)
+# Set an HTTP header in the HTTP response in error context
+# @param $r Current request
+# @param $h Name of the header
+# @param $v Value of the header
 sub lmSetErrHeaderOut {
     my ( $r, $h, $v ) = @_;
     if ( MP() == 2 ) {
@@ -178,6 +264,11 @@ sub lmSetErrHeaderOut {
     }
 }
 
+## @rfn void lmSetHeaderOut(Apache2::RequestRec r, string h, string v)
+# Set an HTTP header in the HTTP response in normal context
+# @param $r Current request
+# @param $h Name of the header
+# @param $v Value of the header
 sub lmSetHeaderOut {
     my ( $r, $h, $v ) = @_;
     if ( MP() == 2 ) {
@@ -188,6 +279,11 @@ sub lmSetHeaderOut {
     }
 }
 
+## @rfn string lmtHeaderOut(Apache2::RequestRec r, string h)
+# Return an HTTP header value from the HTTP response.
+# @param $r Current request
+# @param $h Name of the header
+# @return Value of the header
 sub lmHeaderOut {
     my ( $r, $h, $v ) = @_;
     if ( MP() == 2 ) {
@@ -198,47 +294,105 @@ sub lmHeaderOut {
     }
 }
 
+# Status daemon creation
+
+## @ifn protected void statusProcess()
+# Launch the status processus.
+sub statusProcess {
+    require IO::Pipe;
+    $statusPipe = IO::Pipe->new;
+    $statusOut  = IO::Pipe->new;
+    if ( my $pid = fork() ) {
+        $statusPipe->writer();
+        $statusOut->reader();
+        $statusPipe->autoflush(1);
+    }
+    else {
+        $statusPipe->reader();
+        $statusOut->writer();
+        my $fdin  = $statusPipe->fileno;
+        my $fdout = $statusOut->fileno;
+        open STDIN, "<&$fdin";
+        open STDOUT, ">&$fdout";
+        my @tmp = ();
+        push @tmp, "-I$_" foreach (@INC);
+        exec 'perl', '-MLemonldap::NG::Handler::Status',
+          @tmp,
+          '-e',
+          '&Lemonldap::NG::Handler::Status::run('
+          . $localStorage . ','
+          . Data::Dumper->new( [$localStorageOptions] )->Terse(1)->Dump
+          . ');';
+    }
+}
+
 ##############################
 # Initialization subroutines #
 ##############################
 
-# Security jail
-$safe = new Safe;
-$safe->share( '&encode_base64', '$datas', '&lmSetHeaderIn', '$apacheRequest' );
+## @imethod protected Safe safe()
+# Build and return the security jail used to compile rules and headers.
+# @return Safe object
+sub safe {
+    my $class = shift;
+    return $safe if($safe);
+    $safe = new Safe;
+    my @t = $customFunctions ? split( /\s+/, $customFunctions ) : ();
+    foreach(@t) {
+        $class->lmLog("Custom function : $_",'debug');
+        my $sub = $_;
+        unless(/::/) {
+            $sub = "$class\::$_";
+        }
+        else {
+            s/^.*:://;
+        }
+        next if($class->can($_));
+        eval "sub $_ {
+            return $sub(\$apacheRequest->uri
+                . ( \$apacheRequest->args ? '?' . \$apacheRequest->args : '' )
+                , \@_)
+            }";
+        $class->lmLog($@,'error')if($@);
+    }
+    $safe->share( '&encode_base64', '$datas', '&lmSetHeaderIn', '$apacheRequest', @t );
+    return $safe;
+}
 
-# init() : by default, it calls localInit and globalInit, but with
-#          a shared configuration, init() is overloaded to call only
-#          localInit; globalInit is called later when the configuration
-#          is loaded.
+## @imethod void init(hashRef args)
+# Calls localInit() and globalInit().
+# @param $args reference to the initialization hash
 sub init($$) {
     my $class = shift;
     $class->localInit(@_);
     $class->globalInit(@_);
 }
 
-# Local storage initialization
+## @imethod void localInit(hashRef args)
+# Call purgeCache() to purge the local cache, launch the status process
+# (statusProcess()) in wanted and launch childInit().
+# @param $args reference to the initialization hash
 sub localInit($$) {
     my ( $class, $args ) = @_;
     if ( $localStorage = $args->{localStorage} ) {
         $localStorageOptions = $args->{localStorageOptions};
         $localStorageOptions->{namespace}          ||= "lemonldap";
         $localStorageOptions->{default_expires_in} ||= 600;
-
-        eval "use $localStorage;";
-        die("Unable to load $localStorage: $@") if ($@);
-
-        # At each Apache (re)start, we've to clear the cache to avoid living
-        # with old datas
-        eval '$refLocalStorage = new '
-          . $localStorage
-          . '($localStorageOptions);';
-        if ( defined $refLocalStorage ) {
-            $refLocalStorage->clear();
-        }
-        else {
-            $class->lmLog( "Unable to clear local cache: $@", 'error' );
-        }
+        $class->purgeCache();
     }
+    if ( $args->{status} ) {
+        statusProcess();
+    }
+    $class->childInit();
+}
+
+## @imethod protected boolean childInit()
+# Indicates to Apache that it has to launch:
+# - initLocalStorage() for each child process (after fork and uid change)
+# - cleanLocalStorage() after each requests
+# @return True
+sub childInit {
+    my $class = shift;
 
     # We don't initialise local storage in the "init" subroutine because it can
     # be used at the starting of Apache and so with the "root" privileges. Local
@@ -248,10 +402,9 @@ sub localInit($$) {
     # performances.
     no strict;
     if ( MP() == 2 ) {
-        Apache->push_handlers(
-            PerlChildInitHandler => sub { return $class->initLocalStorage( $_[1], $_[0] ); }
-        );
-        Apache->push_handlers(
+        Apache2::ServerUtil->server->push_handlers( PerlChildInitHandler =>
+              sub { return $class->initLocalStorage( $_[1], $_[0] ); } );
+        Apache2::ServerUtil->server->push_handlers(
             PerlCleanupHandler => sub { return $class->cleanLocalStorage(@_); }
         );
     }
@@ -266,7 +419,35 @@ sub localInit($$) {
     1;
 }
 
-# Global initialization process :
+## @imethod protected void purgeCache()
+# Purge the local cache.
+# Launched at Apache startup.
+sub purgeCache {
+    my $class = shift;
+    eval "use $localStorage;";
+    die("Unable to load $localStorage: $@") if ($@);
+
+    # At each Apache (re)start, we've to clear the cache to avoid living
+    # with old datas
+    eval '$refLocalStorage = new '
+      . $localStorage
+      . '($localStorageOptions);';
+    if ( defined $refLocalStorage ) {
+        $refLocalStorage->clear();
+    }
+    else {
+        $class->lmLog( "Unable to clear local cache: $@", 'error' );
+    }
+}
+
+## @imethod void globalInit(hashRef args)
+# Global initialization process. Launch :
+# - locationRulesInit()
+# - defaultValuesInit()
+# - portalInit()
+# - globalStorageInit()
+# - forgeHeadersInit()
+# @param $args reference to the configuration hash
 sub globalInit {
     my $class = shift;
     $class->locationRulesInit(@_);
@@ -276,12 +457,17 @@ sub globalInit {
     $class->forgeHeadersInit(@_);
 }
 
-# locationRulesInit : used to pre-compile rules :
-#   - rules are stored in a hash containing regexp=>test expressions where :
-#       - regexp is used to test URIs
-#       - test contains an expression used to grant the user
-
-# TODO: split locationRules into 2 arrays
+## @imethod protected void locationRulesInit(hashRef args)
+# Compile rules.
+# Rules are stored in $args->{locationRules} that contains regexp=>test
+# expressions where :
+# - regexp is used to test URIs
+# - test contains an expression used to grant the user
+#
+# This function creates 2 arrays containing :
+# - the list of the compiled regular expressions
+# - the list of the compiled functions (compiled with conditionSub())
+# @param $args reference to the configuration hash
 sub locationRulesInit {
     my ( $class, $args ) = @_;
     $locationCount = 0;
@@ -306,24 +492,27 @@ sub locationRulesInit {
     1;
 }
 
-# conditionSub returns a pre-compiled subroutine used to grant users (used by
+## @imethod protected codeRef conditionSub(string cond)
+# Returns a compiled function used to grant users (used by
 # locationRulesInit().
+# @param $cond The boolean expression to use
 sub conditionSub {
     my ( $class, $cond ) = @_;
     return sub { 1 }
       if ( $cond =~ /^accept$/i );
     return sub { 0 }
       if ( $cond =~ /^deny$/i );
-    if ( $cond =~ /^logout(?:_sso)?(?:\s+(.*))?$/i ) {
-        my $url = $1 || $class->encodeUrl ( "/" );
-        return sub { $logout = $url; return 0 }
+    if ( $cond =~ /^logout(?:_sso)?(?:\s+(.*))$/i ) {
+        my $url = $1;
+        return sub { $datas->{_logout} = $url; return 0 }
     }
-    if( MP() == 2 ) {
+    if ( MP() == 2 ) {
         if ( $cond =~ /^logout_app(?:\s+(.*))?$/i ) {
             my $u = $1;
-            eval 'use Apache2::Filter' unless($INC{"Apache2/Filter.pm"});
+            eval 'use Apache2::Filter' unless ( $INC{"Apache2/Filter.pm"} );
             return sub {
-                $apacheRequest->add_output_filter(sub {
+                $apacheRequest->add_output_filter(
+                    sub {
                         return $class->redirectFilter( $u, @_ );
                     }
                 );
@@ -331,13 +520,18 @@ sub conditionSub {
             };
         }
         elsif ( $cond =~ /^logout_app_sso(?:\s+(.*))?$/i ) {
-            eval 'use Apache2::Filter' unless($INC{"Apache2/Filter.pm"});
-            my $u = encode_base64($1);
-            $u =~ s/[\r\n]//g;
+            eval 'use Apache2::Filter' unless ( $INC{"Apache2/Filter.pm"} );
+            my $u = $1;
             return sub {
                 $class->localUnlog;
-                $apacheRequest->add_output_filter(sub {
-                        return $class->redirectFilter( "$portal?url=$u&logout=1", @_ );
+                $apacheRequest->add_output_filter(
+                    sub {
+                        return $class->redirectFilter(
+                            "$portal?url="
+                              . $class->encodeUrl($u)
+                              . "&logout=1",
+                            @_
+                        );
                     }
                 );
                 1;
@@ -347,11 +541,13 @@ sub conditionSub {
     $cond =~ s/\$date/&POSIX::strftime("%Y%m%d%H%M%S",localtime())/e;
     $cond =~ s/\$(\w+)/\$datas->{$1}/g;
     my $sub;
-    $sub = $safe->reval("sub {return ( $cond )}");
+    $sub = $class->safe->reval("sub {return ( $cond )}");
     return $sub;
 }
 
-# defaultValuesInit : set default values for non-customized variables
+## @imethod protected void defaultValuesInit(hashRef args)
+# Set default values for non-customized variables
+# @param $args reference to the configuration hash
 sub defaultValuesInit {
     my ( $class, $args ) = @_;
 
@@ -363,17 +559,21 @@ sub defaultValuesInit {
     $https = $args->{https} unless defined($https);
     $https = 1 unless defined($https);
     $port = $args->{port} || 0 unless defined($port);
+    $customFunctions = $args->{customFunctions};
     1;
 }
 
-# portalInit : verify that portal variable exists
+## @imethod protected void portalInit(hashRef args)
+# Verify that portal variable exists. Die unless
+# @param $args reference to the configuration hash
 sub portalInit {
     my ( $class, $args ) = @_;
     $portal = $args->{portal} or die("portal parameter required");
 }
 
-# globalStorageInit : initialize the Apache::Session::* package used to
-# share user's variables
+## @imethod protected void globalStorageInit(hashRef args)
+# Initialize the Apache::Session::* module choosed to share user's variables.
+# @param $args reference to the configuration hash
 sub globalStorageInit {
     my ( $class, $args ) = @_;
     $globalStorage = $args->{globalStorage} or die "globalStorage required";
@@ -382,9 +582,10 @@ sub globalStorageInit {
     $globalStorageOptions = $args->{globalStorageOptions};
 }
 
-# forgeHeadersInit : create the &$forgeHeaders subroutine used to insert
-# headers into the HTTP request (which are used for accounting by the
-# application)
+## @imethod protected void forgeHeadersInit(hashRef args)
+# Create the &$forgeHeaders subroutine used to insert
+# headers into the HTTP request.
+# @param $args reference to the configuration hash
 sub forgeHeadersInit {
     my ( $class, $args ) = @_;
 
@@ -407,46 +608,86 @@ sub forgeHeadersInit {
           "lmSetHeaderIn(\$apacheRequest,'$_' => join('',split(/[\\r\\n]+/,"
           . $tmp{$_} . ")));";
     }
-
-    #$sub = "\$forgeHeaders = sub {$sub};";
-    #eval "$sub";
-    $forgeHeaders = $safe->reval("sub {$sub};");
+    $forgeHeaders = $class->safe->reval("sub {$sub};");
     $class->lmLog( "$class: Unable to forge headers: $@: sub {$sub}", 'error' )
       if ($@);
     1;
 }
 
-################
-# MAIN PROCESS #
-################
+## @imethod protected int initLocalStorage()
+# Prepare local cache (if not done before by Lemonldap::NG::Common::Conf)
+# @return Apache2::Const::DECLINED
+sub initLocalStorage {
+    my ( $class, $r ) = @_;
+    if ( $localStorage and not $refLocalStorage ) {
+        eval "use $localStorage;\$refLocalStorage = new $localStorage(\$localStorageOptions);";
+        $class->lmLog( "Local cache initialization failed: $@", 'error' )
+          unless ( defined $refLocalStorage );
+    }
+    return DECLINED;
+}
 
-# grant : grant or refuse client
+###################
+# RUNNING METHODS #
+###################
+
+## @rmethod protected void updateStatus(string user,string url,string action)
+# Inform the status process of the result of the request if it is available.
+sub updateStatus {
+    my ( $class, $user, $url, $action ) = @_;
+    eval {
+        print $statusPipe "$user => "
+          . $apacheRequest->hostname
+          . "$url $action\n"
+          if ($statusPipe);
+    };
+}
+
+## @rmethod protected boolean grant()
+# Grant or refuse client using compiled regexp and functions
+# @return True if the user is granted to access to the current URL
 sub grant {
     my ( $class, $uri ) = @_;
     for ( my $i = 0 ; $i < $locationCount ; $i++ ) {
         return &{ $locationCondition->[$i] }($datas)
           if ( $uri =~ $locationRegexp->[$i] );
     }
-    return &$defaultCondition;
+    return &$defaultCondition($datas);
 }
 
-# forbidden : used to reject non authorizated requests
+## @rmethod protected int forbidden()
+# Used to reject non authorizated requests.
+# Inform the status processus and call logForbidden().
+# @return Apache2::Const::FORBIDDEN
 sub forbidden {
     my $class = shift;
-    if( $logout ) {
-        $apacheRequest->headers_out->set(
-                'Location' => "$portal?url=$logout"
-        );
-        return REDIRECT;
+    if ( $datas->{_logout} ) {
+        $class->updateStatus( $datas->{$whatToTrace}, $_[0], 'LOGOUT' );
+        my $u = $datas->{_logout};
+        $class->localUnlog;
+        return $class->goToPortal( $u, 'logout=1' );
     }
-    $class->lmLog(
-        'The user "' . $datas->{$whatToTrace} . '" was reject when he tried to access to ' . shift,
-        'notice'
-    );
+    $class->updateStatus( $datas->{$whatToTrace}, $_[0], 'REJECT' );
+    $class->logForbidden(@_);
     return FORBIDDEN;
 }
 
-# hideCookie : hide Lemonldap::NG cookie to the protected application
+## @rmethod protected void logForbidden()
+# Insert a log in Apache errors log system to inform that the user was rejected.
+# This method has to be overloaded to use different logs systems
+sub logForbidden {
+    my $class = shift;
+    $class->lmLog(
+        'The user "'
+          . $datas->{$whatToTrace}
+          . '" was reject when he tried to access to '
+          . shift,
+        'notice'
+    );
+}
+
+## @rmethod protected void hideCookie()
+# Hide Lemonldap::NG cookie to the protected application.
 sub hideCookie {
     my $class = shift;
     $class->lmLog( "$class: removing cookie", 'debug' );
@@ -455,25 +696,34 @@ sub hideCookie {
     lmSetHeaderIn( $apacheRequest, 'Cookie' => $tmp );
 }
 
+## @rmethod protected string encodeUrl(string url)
+# Encode URl in the format used by Lemonldap::NG::Portal for redirections.
 sub encodeUrl {
     my ( $class, $url ) = @_;
-    my $portString = $port || $apacheRequest->get_server_port();
-    $portString =
-        (  $https && $port == 443 ) ? ''
-      : ( !$https && $port == 80 )  ? ''
-      :                               ':' . $portString;
-    my $u =
-      encode_base64( "http"
+    my $u = $url;
+    if ( $url !~ m#^https?://# ) {
+        my $portString = $port || $apacheRequest->get_server_port();
+        $portString =
+            ( $https  && $portString == 443 ) ? ''
+          : ( !$https && $portString == 80 )  ? ''
+          :                                     ':' . $portString;
+        $u = "http"
           . ( $https ? "s" : "" ) . "://"
           . $apacheRequest->get_server_name()
           . $portString
-          . $url );
+          . $url;
+    }
+    $u = encode_base64($u);
     $u =~ s/[\r\n\s]//sg;
     return $u;
 }
 
-# Redirect non-authenticated users to the portal
-sub goToPortal() {
+## @rmethod protected int goToPortal(string url, string arg)
+# Redirect non-authenticated users to the portal by setting "Location:" header.
+# @param $url Url requested
+# @param $arg optionnal GET parameters
+# @return Apache2::Const::REDIRECT
+sub goToPortal {
     my ( $class, $url, $arg ) = @_;
     $class->lmLog(
         "Redirect "
@@ -481,32 +731,48 @@ sub goToPortal() {
           . " to portal (url was $url)",
         'debug'
     );
-    my $urlc_init = $class->encodeUrl ( $url );
-    $apacheRequest->headers_out->set(
-        'Location' => "$portal?url=$urlc_init" . ( $arg ? "&$arg" : "" )
-    );
+    my $urlc_init = $class->encodeUrl($url);
+    lmSetHeaderOut( $apacheRequest,
+        'Location' => "$portal?url=$urlc_init" . ( $arg ? "&$arg" : "" ) );
     return REDIRECT;
 }
 
-# Fetch $id
-sub fetchId() {
+## @rmethod protected $ fetchId()
+# Get user cookies and search for Lemonldap::NG cookie.
+# @return Value of the cookie if found, 0 else
+sub fetchId {
     my $t = lmHeaderIn( $apacheRequest, 'Cookie' );
-    return ($t =~ /$cookieName=([^; ]+);?/o ) ? $1: 0;
+    return ( $t =~ /$cookieName=([^; ]+);?/o ) ? $1 : 0;
 }
 
 # MAIN SUBROUTINE called by Apache (using PerlHeaderParserHandler option)
+
+## @rmethod int run(Apache2::RequestRec apacheRequest)
+# Main method used to control access.
+# Calls :
+# - fetchId()
+# - lmSetApacheUser()
+# - grant()
+# - forbidden() if user is rejected
+# - sendHeaders() if user is granted
+# - hideCookie()
+# - updateStatus()
+# @param $apacheRequest Current request
+# @return Apache2::Const value (OK, FORBIDDEN, REDIRECT or SERVER_ERROR)
 sub run ($$) {
     my $class;
     ( $class, $apacheRequest ) = @_;
 
     return DECLINED unless ( $apacheRequest->is_initial_req );
-    my $uri = $apacheRequest->uri . ( $apacheRequest->args ? "?" . $apacheRequest->args : "" );
+    my $uri = $apacheRequest->uri
+      . ( $apacheRequest->args ? "?" . $apacheRequest->args : "" );
 
     # AUTHENTICATION
     # I - recover the cookie
     my $id;
     unless ( $id = $class->fetchId ) {
         $class->lmLog( "$class: No cookie found", 'info' );
+        $class->updateStatus( $apacheRequest->connection->remote_ip, $apacheRequest->uri, 'REDIRECT' );
         return $class->goToPortal($uri);
     }
 
@@ -526,6 +792,7 @@ sub run ($$) {
                 # The cookie isn't yet available
                 $class->lmLog( "The cookie $id isn't yet available: $@",
                     'info' );
+                $class->updateStatus( $apacheRequest->connection->remote_ip, $apacheRequest->uri, 'EXPIRED' );
                 return $class->goToPortal($uri);
             }
             $datas->{$_} = $h{$_} foreach ( keys %h );
@@ -540,10 +807,11 @@ sub run ($$) {
 
     # ACCOUNTING
     # 1 - Inform Apache
-    $apacheRequest->connection->user( $datas->{$whatToTrace} ) if( $datas->{$whatToTrace} );
+    lmSetApacheUser( $apacheRequest, $datas->{$whatToTrace} );
 
     # AUTHORIZATION
     return $class->forbidden($uri) unless ( $class->grant($uri) );
+    $class->updateStatus( $datas->{$whatToTrace}, $apacheRequest->uri, 'OK' );
     $class->lmLog(
         "User "
           . $datas->{$whatToTrace}
@@ -561,68 +829,129 @@ sub run ($$) {
     OK;
 }
 
+## @rmethod protected void sendHeaders()
+# Launch function compiled by forgeHeadersInit()
 sub sendHeaders {
     &$forgeHeaders;
 }
 
-sub initLocalStorage {
-    my ( $class, $r ) = @_;
-    if ( $localStorage and not $refLocalStorage ) {
-        eval '$refLocalStorage = new '
-          . $localStorage
-          . '($localStorageOptions);';
-    }
-    $class->lmLog( "Local cache initialization failed: $@", 'error' )
-      unless ( defined $refLocalStorage );
-    return DECLINED;
-}
-
-sub cleanLocalStorage {
-    $refLocalStorage->purge() if ($refLocalStorage);
-    return DECLINED;
-}
-
+## @rmethod int unprotect()
+# Used to unprotect an area.
+# To use it, set "PerlHeaderParserHandler My::Package->unprotect" Apache
+# configuration file.
+# It replace run() by doing nothing.
+# @return Apache2::Const::OK
 sub unprotect {
     OK;
 }
 
+## @rmethod protected void localUnlog()
+# Delete current user from local cache entry.
 sub localUnlog {
     my $class = shift;
-    if( my $id = $class->fetchId ) {
+    if ( my $id = $class->fetchId ) {
+
         # Delete Apache thread datas
         if ( $id eq $datas->{_session_id} ) {
             $datas = {};
         }
+
         # Delete Apache local cache
-        if( $refLocalStorage and $refLocalStorage->get($id) ) {
+        if ( $refLocalStorage and $refLocalStorage->get($id) ) {
             $refLocalStorage->remove($id);
         }
     }
 }
 
+## @rmethod protected int unlog(Apache::RequestRec apacheRequest)
+# Call localUnlog() then goToPortal() to unlog the current user.
+# @return Apache2::Const value returned by goToPortal()
 sub unlog ($$) {
     my $class;
-    $logout = 0;
-    ($class, $apacheRequest ) = @_;
+    ( $class, $apacheRequest ) = @_;
     $class->localUnlog;
+    $class->updateStatus( $apacheRequest->connection->remote_ip, $apacheRequest->uri, 'LOGOUT' );
     return $class->goToPortal( '/', 'logout=1' );
 }
 
+## @rmethod protected int redirectFilter(string url, Apache2::Filter f)
+# Launch the current HTTP request then redirects the user to $url.
+# Used by logout_app and logout_app_sso targets
+# @param $url URL to redirect the user
+# @param $f Current Apache2::Filter object
+# @return Apache2::Const::REDIRECT
 sub redirectFilter {
     my $class = shift;
     my $url   = shift;
     my $f     = shift;
-    unless ($f->ctx) {
+    unless ( $f->ctx ) {
+
+        # Here, we can use Apache2 functions instead of lmSetHeaderOut because
+        # this function is used only with Apache2.
         $f->r->status(REDIRECT);
         $f->r->status_line("302 Temporary Moved");
-        $f->r->err_headers_out->set('Location' => $url);
+        $f->r->headers_out->unset('Location');
+        $f->r->err_headers_out->set( 'Location' => $url );
         $f->ctx(1);
     }
-    while ($f->read(my $buffer, 1024)) {
+    while ( $f->read( my $buffer, 1024 ) ) {
     }
+    $class->updateStatus( ( $datas->{$whatToTrace} ? $datas->{$whatToTrace} : $f->r->connection->remote_ip ), 'filter', 'REDIRECT' );
     return REDIRECT;
 }
 
+## @rmethod int status(Apache2::RequestRec $r)
+# Get the result from the status process and launch a PerlResponseHandler to
+# display it.
+# @param $r Current request
+# @return Apache2::Const::OK
+sub status($$) {
+    my ( $class, $r ) = @_;
+    $class->lmLog( "$class: request for status", 'debug' );
+    return SERVER_ERROR unless ( $statusPipe and $statusOut );
+    $r->handler("perl-script");
+    print $statusPipe "STATUS" . ( $r->args ? " " . $r->args : '' ) . "\n";
+    my $buf;
+    while (<$statusOut>) {
+        last if (/^END$/);
+        $buf .= $_;
+    }
+    if ( MP() == 2 ) {
+        $r->push_handlers(
+            'PerlResponseHandler' => sub {
+                my $r = shift;
+                $r->content_type('text/html; charset=UTF-8');
+                $r->print($buf);
+                OK;
+            }
+        );
+    }
+    else {
+        $r->push_handlers(
+            'PerlHandler' => sub {
+                my $r = shift;
+                $r->content_type('text/html; charset=UTF-8');
+                $r->send_http_header;
+                $r->print($buf);
+                OK;
+            }
+        );
+    }
+    return OK;
+}
+
+#################
+# OTHER METHODS #
+#################
+
+## @rmethod protected int cleanLocalStorage()
+# Clean expired values from the local cache.
+# @return Apache2::Const::DECLINED
+sub cleanLocalStorage {
+    $refLocalStorage->purge() if ($refLocalStorage);
+    return DECLINED;
+}
+
 1;
 __END__
 
@@ -667,9 +996,9 @@ More complete example
   our @ISA = qw(Lemonldap::NG::Handler::Simple);
 
   __PACKAGE__->init ( { locationRules => {
-             '^/pj/.*$'       => q($qualif="opj"),
-             '^/rh/.*$'       => q($ou=~/brh/),
-             '^/rh_or_opj.*$' => q($qualif="opj or $ou=~/brh/),
+             '^/pj/.*$'       => '$qualif="opj"',
+             '^/rh/.*$'       => '$ou=~/brh/',
+             '^/rh_or_opj.*$' => '$qualif="opj" or $ou=~/brh/',
              default => 'accept', # means that all authenticated users are greanted
            },
            globalStorage        => 'Apache::Session::MySQL',
